/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.QuadCurve2D;

import org.gfxtest.framework.*;
import org.gfxtest.framework.annotations.*;



/**
 * This test renders various quadratic curves using identity transformation
 * matrix. Quadratic curves are constructed using QuadCurve2D class. Curves
 * are drawn using various colors and stroke width.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.RENDER_TEST)
@GraphicsPrimitive(GraphicsPrimitives.SHAPE)
@ShapeVariant(ShapeVariants.QUADRATIC_CURVE)
@RenderStyle(RenderStyles.NORMAL)
@Transformation(Transformations.NONE)
@Zoom(1)
public class NormalQuadraticCurves extends GfxTest
{
    /**
     * Default Y offset of curve end points.
     */
    private static final int DEFAULT_Y_OFFSET = 40;

    /**
     * Step between curves drawn by different colors onto the same image.
     */
    private static final int OFFSET_STEP = 20;

    /**
     * Draw crosses at the both end points and control point of quadratic curve.
     * 
     * @param graphics
     *            graphics canvas
     * @param x1
     *            first quadratic curve x coordinate
     * @param y1
     *            first quadratic curve y coordinate
     * @param x2
     *            second quadratic curve x coordinate
     * @param y2
     *            second quadratic curve y coordinate
     * @param x3
     *            third quadratic curve x coordinate
     * @param y3
     *            third quadratic curve x coordinate
     */
    private void drawCrossesAtControlPoints(Graphics2D graphics, int x1, int y1, int x2, int y2, int x3, int y3)
    {
        drawCross(graphics, x1, y1);
        drawCross(graphics, x2, y2);
        drawCross(graphics, x3, y3);
    }

    /**
     * Draw quadratic curve onto canvas specified by Graphics2D class.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics canvas
     * @param quadCurve
     *            curve to be drawn
     * @param color
     *            curve color or null when Color.BLACK can be used.
     * @return test result status - PASSED, FAILED or ERROR
     */
    private TestResult drawQuadraticCurve(TestImage image, Graphics2D graphics, QuadCurve2D quadCurve, Color color)
    {
        // calculate image dimensions
        int width =  image.getWidth();
        int height =  image.getHeight();

        // get curve coordinates
        int x1 = computeX1(width);
        int y1 = computeY1();
        int x2 = computeX2(width);
        int y2 = computeY2(height);
        int x3 = computeX3(width);
        int y3 = computeY3();

        // construct QuadCurve2D.Float with set coordinates
        quadCurve.setCurve(x1, y1, x2, y2, x3, y3);

        // set the specified color
        graphics.setColor(color == null ? Color.BLACK : color);

        // draw QuadCurve2D
        graphics.draw(quadCurve);

        // draw crosses at all control points
        drawCrossesAtControlPoints(graphics, x1, y1, x2, y2, x3, y3);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Draw quadratic curve onto canvas specified by Graphics2D class using
     * black color.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics canvas
     * @param quadCurve
     *            curve to be drawn
     * @return test result status - PASSED, FAILED or ERROR
     */
    private TestResult drawQuadraticCurve(TestImage image, Graphics2D graphics, QuadCurve2D quadCurve)
    {
        return drawQuadraticCurve(image, graphics, quadCurve, Color.BLACK);
    }
    
    /**
     * Draw set of quadratic curves onto canvas specified by Graphics2D class.
     * Curves are drawn using various colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics canvas
     */
    private void drawCurveWithVariousColors(TestImage image, Graphics2D graphics)
    {
        // calculate image dimensions
        int width =  image.getWidth();
        int height =  image.getHeight();

        // curve colors
        Color[] colors = ColorPalette.getColors();

        // offset counter
        int offset = DEFAULT_Y_OFFSET;

        // Draw each curve with different color.
        for (Color color : colors)
        {
            // create new QuadCurve2D.Float
            QuadCurve2D quadCurve = new QuadCurve2D.Float();

            // construct QuadCurve2D.Float with set coordinates
            quadCurve.setCurve(computeX1(width), offset, computeX2(width), height * 2 / 3 + offset, computeX3(width),
                            offset);

            // set the specified color
            graphics.setColor(color);

            // draw QuadCurve2D
            graphics.draw(quadCurve);

            // move next curve down
            offset += OFFSET_STEP;
        }
    }

    /**
     * Draw set of quadratic curves onto canvas specified by Graphics2D class.
     * Curves are drawn using various transparency.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics canvas
     */
    private void drawCurveWithVariousTransparency(TestImage image, Graphics2D graphics)
    {
        // calculate image dimensions
        int width =  image.getWidth();
        int height =  image.getHeight();

        // offset counter
        int offset = DEFAULT_Y_OFFSET;

        // Draw each curve with transparency
        for (int transparency = 0; transparency < 16; transparency++)
        {
            // create new QuadCurve2D.Float
            QuadCurve2D quadCurve = new QuadCurve2D.Float();

            // construct QuadCurve2D.Float with set coordinates
            quadCurve.setCurve(computeX1(width), offset, computeX2(width), height * 2 / 3 + offset, computeX3(width),
                            offset);

            // set the specified color
            graphics.setColor(new Color(0f, 0f, 0f, transparency / 16f));

            // draw QuadCurve2D
            graphics.draw(quadCurve);

            // move next curve down
            offset += OFFSET_STEP;
        }
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBasicQuadCurveFloat(TestImage image, Graphics2D graphics)
    {
        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // draw quadratic curve
        return drawQuadraticCurve(image, graphics, quadCurve);
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Double() is rendered
     * correctly.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBasicQuadCurveDouble(TestImage image, Graphics2D graphics)
    {
        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Double();

        // draw quadratic curve
        return drawQuadraticCurve(image, graphics, quadCurve);
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with zero pixels wide stroke and default
     * caps.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testZeroStroke(TestImage image, Graphics2D graphics)
    {
        // set zero pixels wide stroke
        CommonRenderingStyles.setStrokeZeroThick(graphics);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // draw quadratic curve
        return drawQuadraticCurve(image, graphics, quadCurve);
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 10 pixels wide stroke and default
     * caps.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testThickStroke(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // draw quadratic curve
        return drawQuadraticCurve(image, graphics, quadCurve);
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 30 pixels wide stroke and default
     * caps.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testExtraThickStroke(TestImage image, Graphics2D graphics)
    {
        // set 30 pixels wide stroke
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // draw quadratic curve
        return drawQuadraticCurve(image, graphics, quadCurve);
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 10 pixels wide stroke and default
     * CAP_BUTT.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testThickStrokeCapsButt(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke and CAP_BUTT
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // draw quadratic curve
        return drawQuadraticCurve(image, graphics, quadCurve);
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 10 pixels wide stroke and default
     * CAP_ROUND.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testThickStrokeCapsRound(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke and CAP_ROUND
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // draw quadratic curve
        return drawQuadraticCurve(image, graphics, quadCurve);
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 10 pixels wide stroke and default
     * CAP_SQUARE.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testThickStrokeCapsSquare(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke and CAP_SQUARE
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // draw quadratic curve
        return drawQuadraticCurve(image, graphics, quadCurve);
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 30 pixels wide stroke and default
     * CAP_BUTT.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testExtraThickStrokeCapsButt(TestImage image, Graphics2D graphics)
    {
        // set 30 pixels wide stroke and CAP_BUTT
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // draw quadratic curve
        return drawQuadraticCurve(image, graphics, quadCurve);
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 30 pixels wide stroke and default
     * CAP_ROUND.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testExtraThickStrokeCapsRound(TestImage image, Graphics2D graphics)
    {
        // set 30 pixels wide stroke and CAP_ROUND
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // draw quadratic curve
        return drawQuadraticCurve(image, graphics, quadCurve);
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 30 pixels wide stroke and default
     * CAP_SQUARE.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testExtraThickStrokeCapsSquare(TestImage image, Graphics2D graphics)
    {
        // set 30 pixels wide stroke and CAP_SQUARE
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // draw quadratic curve
        return drawQuadraticCurve(image, graphics, quadCurve);
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is drawn using different colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testColors(TestImage image, Graphics2D graphics)
    {
        // draw set of curves using test color palette
        drawCurveWithVariousColors(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is drawn using different colors and using 10 pixel wide
     * stroke.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testColorsStrokeWidth(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics); 

        // draw set of curves using test color palette
        drawCurveWithVariousColors(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is drawn using different colors and using zero pixel
     * wide stroke.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testColorsZeroStrokeWidth(TestImage image, Graphics2D graphics)
    {
        // set zero pixels wide stroke
        CommonRenderingStyles.setStrokeZeroThick(graphics); 

        // draw set of curves using test color palette
        drawCurveWithVariousColors(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is drawn using different colors and using 10 pixel wide
     * stroke using CAP_BUTT.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testColorsCapsButt(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);

        // draw set of curves using test color palette
        drawCurveWithVariousColors(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is drawn using different colors and using 10 pixel wide
     * stroke using CAP_ROUND.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testColorsCapsRound(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);

        // draw set of curves using test color palette
        drawCurveWithVariousColors(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered
     * correctly. Curve is drawn using different colors and using 10 pixel wide
     * stroke using CAP_SQUARE.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testColorsCapsSquare(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL);

        // draw set of curves using test color palette
        drawCurveWithVariousColors(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered correctly.
     * Curve is drawn using different transparency.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testTransparencyBasicStroke(TestImage image, Graphics2D graphics)
    {
        // draw set of curves using different transparency
        drawCurveWithVariousTransparency(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered correctly.
     * Curve is drawn using different transparency and using 10 pixel wide
     * stroke.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testTransparencyThickStroke(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics);

        // draw set of curves using different transparency
        drawCurveWithVariousTransparency(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered correctly.
     * Curve is drawn using different transparency and using 30 pixel wide
     * stroke.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testTransparencyExtraThickStroke(TestImage image, Graphics2D graphics)
    {
        // set 30 pixels wide stroke
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics);

        // draw set of curves using different transparency
        drawCurveWithVariousTransparency(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if quadratic curve created by QuadCurve2D.Float() is rendered correctly.
     * Curve is drawn using different transparency and using 30 pixel wide
     * stroke.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testTransparencyZeroThickStroke(TestImage image, Graphics2D graphics)
    {
        // set zero pixels wide stroke
        CommonRenderingStyles.setStrokeZeroThick(graphics);

        // draw set of curves using different transparency
        drawCurveWithVariousTransparency(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test special test case - quadratic curve with no identical control points.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testNoIdenticalPoints(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth();
        int height = image.getHeight();

        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL);

        // set curve color
        graphics.setColor(Color.BLACK);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // get curve coordinates
        int x1 = computeX1(width);
        int y1 = computeY1();
        int x2 = computeX2(width);
        int y2 = computeY2(height);
        int x3 = computeX3(width);
        int y3 = computeY3();

        // construct QuadCurve2D.Float with set coordinates
        quadCurve.setCurve(x1, y1, x2, y2, x3, y3);

        // draw QuadCurve2D
        graphics.draw(quadCurve);

        // draw crosses at all control points
        drawCrossesAtControlPoints(graphics, x1, y1, x2, y2, x3, y3);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test special test case - quadratic curve with identical first point and
     * second point. To be precise: first point is curve end point, second point
     * is control point which defines the curvature.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints12(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth();
        int centerY = image.getCenterY();

        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL);

        // set curve color
        graphics.setColor(Color.BLACK);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // get curve coordinates
        int x1 = computeX1(width);
        int y1 = centerY;
        int x2 = x1;
        int y2 = y1;
        int x3 = computeX3(width);
        int y3 = centerY;

        // construct QuadCurve2D.Float with set coordinates
        quadCurve.setCurve(x1, y1, x2, y2, x3, y3);

        // draw QuadCurve2D
        graphics.draw(quadCurve);

        // draw crosses at all control points
        drawCrossesAtControlPoints(graphics, x1, y1, x2, y2, x3, y3);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test special test case - quadratic curve with identical second point and
     * third point. To be precise: third point is curve end point, second point
     * is control point which defines the curvature.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints23(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth();
        int centerY = image.getCenterY();

        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL);

        // set curve color
        graphics.setColor(Color.BLACK);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // get curve coordinates
        int x1 = computeX1(width);
        int y1 = centerY;
        int x2 = computeX3(width);
        int y2 = y1;
        int x3 = x2;
        int y3 = y1;

        // construct QuadCurve2D.Float with set coordinates
        quadCurve.setCurve(x1, y1, x2, y2, x3, y3);

        // draw QuadCurve2D
        graphics.draw(quadCurve);

        // draw crosses at all control points
        drawCrossesAtControlPoints(graphics, x1, y1, x2, y2, x3, y3);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test special test case - quadratic curve with identical first point and
     * third point.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints13(TestImage image, Graphics2D graphics)
    {
        int width = image.getWidth();
        int centerY = image.getCenterY();

        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL);

        // set curve color
        graphics.setColor(Color.BLACK);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // get curve coordinates
        int x1 = computeX1(width);
        int y1 = centerY;
        int x2 = computeX3(width);
        int y2 = y1;
        int x3 = x1;
        int y3 = y1;

        // construct QuadCurve2D.Float with set coordinates
        quadCurve.setCurve(x1, y1, x2, y2, x3, y3);

        // draw QuadCurve2D
        graphics.draw(quadCurve);

        // draw crosses at all control points
        drawCrossesAtControlPoints(graphics, x1, y1, x2, y2, x3, y3);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test special test case - quadratic curve with all identical points.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints123(TestImage image, Graphics2D graphics)
    {
        int centerX = image.getCenterX();
        int centerY = image.getCenterY();

        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL);

        // set curve color
        graphics.setColor(Color.BLACK);

        // create new QuadCurve2D.Float
        QuadCurve2D quadCurve = new QuadCurve2D.Float();

        // construct QuadCurve2D.Float with set coordinates
        quadCurve.setCurve(centerX, centerY, centerX, centerY, centerX, centerY);

        // draw QuadCurve2D
        graphics.draw(quadCurve);

        // draw crosses at all control points
        drawCrossesAtControlPoints(graphics, centerX, centerY, centerX, centerY, centerX, centerY);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Compute X coordinate of first curve end point.
     * 
     * @param width
     *            canvas width
     * @return X coordinate of first curve end point.
     */
    private int computeX1(int width)
    {
        return width >> 2;
    }

    /**
     * Compute X coordinate of curve control point.
     *
     * @param width
     *            canvas width
     * @return X coordinate of curve control point.
     */
    private int computeX2(int width)
    {
        return width >> 1;
    }

    /**
     * Compute X coordinate of second curve end point.
     * 
     * @param width
     *            canvas width
     * @return X coordinate of second curve end point.
     */
    private int computeX3(int width)
    {
        return width * 3 / 4;
    }

    /**
     * Compute Y coordinate of first curve end point.
     *
     * @return Y coordinate of first curve end point.
     */
    private int computeY1()
    {
        return DEFAULT_Y_OFFSET;
    }

    /**
     * Compute Y coordinate of curve control point.
     *
     * @param height
     *            canvas height
     * @return Y coordinate of curve control point.
     */
    private int computeY2(int height)
    {
        return DEFAULT_Y_OFFSET + height * 2 / 3;
    }

    /**
     * Compute Y coordinate of second curve end point.
     *
     * @return Y coordinate of second curve end point.
     */
    private int computeY3()
    {
        return DEFAULT_Y_OFFSET;
    }

    /**
     * Entry point to the test suite.
     *
     * @param args not used in this case
     */
    public static void main(String[] args)
    {
        new NormalQuadraticCurves().runTestSuite(args);
    }
}
